#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usb.h"

#define CPU_PRESCALE(n)	(CLKPR = 0x80, CLKPR = (n))
#define CPU_16MHz	0x00
#define CPU_125kHz	0x07

volatile uint8_t led_count=0;
volatile uint8_t pb_count=0;

uint8_t left(void);
uint8_t right(void);
void led(unsigned char c);


#define print(s) print_P(PSTR(s))
void print_P(const char *s)
{
	char c;
	while (1) {
		c = pgm_read_byte(s++);
		if (!c) break;
		usb_serial_putchar(c);
	}
}

#define DTR() (usb_serial_get_control() & USB_SERIAL_DTR)

void welcome_message(void)
{
	static uint8_t state=0;
	uint8_t now;


	now = DTR();
	if (now == state) return;
	state = now;
	if (!now) return;
	print("Welcome to Paul's USB Composite Device Demo\r\n");
	print("using Interface Association Descriptors\r\n");
	print("\r\n");
	print("  1: Communication Class ACM (Serial)\r\n");
	print("  2: HID Class (Mouse)\r\n");
	print("  3: Mass Storage Class (Disk)\r\n");
	print("\r\n");
	print("Any characters sent to Serial will be echoed.\r\n");
	print("Pressing the pushbuttons will send mouse movements\r\n");
	print("AND messages to this serial inferface.  The Disk\r\n");
	print("interface does not interface with Serial, but you\r\n");
	print("can access the files stored in this firmware image.\n\n");
	print("\r\n");
	print("If you have any questions, please contact me at\r\n");
	print("paul@pjrc.com or call 503-625-9328.  I hope you find\r\n");
	print("this little demo useful.  :-)\r\n");
	print("\r\n");
	return;
}

int main(void)
{
	//uint16_t val, count=1;
	//uint8_t i;
	int16_t c;

	CPU_PRESCALE(CPU_125kHz);
	_delay_ms(1);		// allow slow power supply startup
	CPU_PRESCALE(CPU_16MHz); // set for 16 MHz clock

	PORTA = 0xFF;
	PORTB = 0xFF;
	PORTC = 0xFF;
	PORTD = 0xFF;
	PORTE = 0xFF;
	PORTF = 0xFF;
	DDRD = (1<<6);
	led(0);

	usb_init();
	_delay_ms(1000);

	// Configure timer 0 to generate a timer overflow interrupt every
        // 256*1024 clock cycles, or approx 61 Hz when using 16 MHz clock
        TCCR0A = 0x00;
        TCCR0B = 0x05;
        TIMSK0 = (1<<TOIE0);

	while (1) {
		welcome_message();

		if (left()) {
			usb_mouse_move(-3, 0, 0);
			led(1);
		}
		if (right()) {
			usb_mouse_move(3, 0, 0);
			led(1);
		}

		c = usb_serial_getchar();
		if (c >= 0) {
			print("char '");
			usb_serial_putchar(c);
			print("'\r\n");
			led(1);
		}
	}
}



void led(unsigned char c)
{
	led_count = 0;
	if (c) PORTD |= (1<<6);
	else PORTD &= ~(1<<6);
}

#define LEFT_BUTTON() (PINC & (1<<7) ? 0 : 1)
#define RIGHT_BUTTON() (PINC & (1<<6) ? 0 : 1)

uint8_t left(void)
{
	static uint8_t state=0;
	uint8_t now;

	if (pb_count < 3) return 0;
	now = LEFT_BUTTON();
	if (now == state) {
		if (now && pb_count > 40) {
			pb_count = 28;
			print("Left Auto-Repeat\r\n");
			return 1;
		}
		return 0;
	}
	pb_count = 0;
	state = now;
	if (now) {
		print("Left Button\r\n");
		led(1);
		return 1;
	}
	return 0;
}

uint8_t right(void)
{
	static uint8_t state=0;
	uint8_t now;

	if (pb_count < 3) return 0;
	now = RIGHT_BUTTON();
	if (now == state) {
		if (now && pb_count > 40) {
			pb_count = 28;
			print("Right Auto-Repeat\r\n");
			return 1;
		}
		return 0;
	}
	pb_count = 0;
	state = now;
	if (now) {
		print("Right Button\r\n");
		led(1);
		return 1;
	}
	return 0;
}



// This interrupt routine is run approx 61 times per second.
ISR(TIMER0_OVF_vect)
{
        if (pb_count < 255) pb_count++;
	if (led_count < 8) led_count++; else led(0);
}

